<?php

use GuzzleHttp\Exception\RequestException;

require_once __DIR__ . '/../vendor/autoload.php';

class TC_PaygateHeleket_Processor_Heleket extends bdPaygate_Processor_Abstract
{
	const CURRENCY_RUB = 'rub';

	/**
	 * @return string
	 */
	public function getFormEndpoint()
	{
		return XenForo_Link::buildPublicLink('tc-pg-heleket/select-currency');
	}

	/**
	 * @return string
	 */
	public function getApiEndpoint()
	{
		return 'https://api.heleket.com/';
	}

	/**
	 * @var string
	 */
	private $cryptoCurrency;

	/**
	 * @var string
	 */
	private $cryptoNetwork;

	/**
	 * @return array|string[]
	 */
	public function getSupportedCurrencies()
	{
		return [
			bdPaygate_Processor_Abstract::CURRENCY_USD,
			bdPaygate_Processor_Abstract::CURRENCY_EUR,
			self::CURRENCY_RUB
		];
	}

	/**
	 * @return bool
	 */
	public function isAvailable()
	{
		$options = XenForo_Application::getOptions();

		return !(
			empty($options->tcPaygateHeleket_MerchantId)
			|| empty($options->tcPaygateHeleket_ApiKey)
			|| $this->_sandboxMode()
		);
	}

	/**
	 * @return bool
	 */
	public function isRecurringSupported()
	{
		return false;
	}

	/**
	 * @param Zend_Controller_Request_Http $request
	 * @param string                       $transactionId
	 * @param string                       $paymentStatus
	 * @param string                       $transactionDetails
	 * @param string                       $itemId
	 *
	 * @return bool
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function validateCallback(
		Zend_Controller_Request_Http $request,
									 &$transactionId,
									 &$paymentStatus,
									 &$transactionDetails,
									 &$itemId
	)
	{
		$amount = false;
		$currency = false;

		return $this->validateCallback2(
			$request,
			$transactionId,
			$paymentStatus,
			$transactionDetails,
			$itemId,
			$amount,
			$currency
		);
	}

	/**
	 * @param Zend_Controller_Request_Http $request
	 * @param string                       $transactionId
	 * @param string                       $paymentStatus
	 * @param string                       $transactionDetails
	 * @param string                       $itemId
	 * @param string                       $amount
	 * @param string                       $currency
	 *
	 * @return bool
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function validateCallback2(
		Zend_Controller_Request_Http $request,
									 &$transactionId,
									 &$paymentStatus,
									 &$transactionDetails,
									 &$itemId,
									 &$amount,
									 &$currency
	)
	{
		$input = @json_decode($request->getRawBody(), true);
		$signature = $input['sign'];

		if (!empty($input))
		{
			$transactionDetails = $input;
			$xfOptions = XenForo_Application::get('options');
			if (!empty($xfOptions->tcPaygateHeleket_MerchantId)
				&& !empty($xfOptions->tcPaygateHeleket_ApiKey))
			{
				$signatureArray = $input;
				unset($signatureArray['sign']);

				$generatedSignature = $this->getSignature(json_encode($signatureArray, JSON_UNESCAPED_UNICODE));
				if (hash_equals($signature, $generatedSignature))
				{
					$transactionId = 'heleket_' . $input['uuid'];

					$amount = $input['amount'];
					$currency = $input['currency'];

					if ($input['is_final'] && ($input['status'] == 'paid' || $input['status'] == 'paid_over'))
					{
						$processorModel = $this->getModelFromCache('bdPaygate_Model_Processor');

						// Проверяем, не была ли уже проведена такая операция
						$log = $processorModel->getLogByTransactionId($transactionId);
						if (!empty($log))
						{
							if (($log['log_type'] == 'accepted') || ($log['log_type'] == 'rejected'))
							{
								$this->_setError("Transaction {$transactionId} has already been processed");

								echo $this->getResponse($this->getLastError(), 'error');

								return false;
							}
						}

						$additionalData = json_decode($input['additional_data'], true);
						$itemId = base64_decode($additionalData['item_id']);

						$paymentStatus = bdPaygate_Processor_Abstract::PAYMENT_STATUS_ACCEPTED;

						echo $this->getResponse('Payment received.');

						return true;
					}

					$this->_setError('Ok. No action.');

					echo $this->getResponse($this->getLastError(), 'info');

					return false;
				}

				$this->_setError('Invalid signature');

				echo $this->getResponse($this->getLastError(), 'error');

				return false;
			}

			$this->_setError('Invalid public_key or secret_key.');

			echo $this->getResponse($this->getLastError(), 'error');

			return false;
		}


		$this->_setError('Callback not validated.');

		echo $this->getResponse($this->getLastError(), 'error');

		return false;
	}

	/**
	 * @param float  $amount
	 * @param string $currency
	 * @param string $itemName
	 * @param string $itemId
	 * @param bool   $recurringInterval
	 * @param bool   $recurringUnit
	 * @param array  $extraData
	 *
	 * @return string
	 * @throws XenForo_Exception
	 * @throws Zend_Exception
	 */
	public function generateFormData(
		$amount,
		$currency,
		$itemName,
		$itemId,
		$recurringInterval = false,
		$recurringUnit = false,
		array $extraData = []
	)
	{
		$this->_assertAmount($amount);
		$this->_assertCurrency($currency);
		$this->_assertItem($itemName, $itemId);
		$this->_assertRecurring($recurringInterval, $recurringUnit);

		$visitor = XenForo_Visitor::getInstance();
		$xfOptions = XenForo_Application::get('options');

		$paymentParams = [
			'amount'       => $amount,
			'currency'     => $currency,
			'return_url'   => $this->_generateReturnUrl($extraData),
			'callback_url' => $this->_generateCallbackUrl($extraData),
			'item'         => base64_encode($itemId)
		];

		$dataString = json_encode($paymentParams, JSON_UNESCAPED_UNICODE);
		$sign = $this->getSignature($dataString, $xfOptions->tcPaygateHeleket_ApiKey);

		// Генерация формы
		$form = "<form action=\"{$this->getFormEndpoint()}\" method=\"POST\" class=\"xenForm formOverlay\">";

		$params = [
			'data'     => $dataString,
			'sign'     => $sign,
			'_xfToken' => $visitor['csrf_token_page']
		];

		$callToAction = new XenForo_Phrase('tc_pg_heleket_call_to_action');

		foreach ($params as $item => $value)
		{
			$value = htmlspecialchars($value);
			$form .= "<input type=\"hidden\" name=\"$item\" value=\"$value\" />";
		}

		$form .= "<input type=\"submit\" value=\"{$callToAction}\" class=\"button OverlayTrigger\"/></form><br />";

		return $form;
	}

	/**
	 * @param string $path
	 * @param array  $data
	 * @param        $error
	 *
	 * @return array
	 */
	public function apiCall($path, array $data, &$error = null, $merchantId = null, $apiKey = null)
	{
		$xfOptions = XenForo_Application::getOptions();
		$merchantId = $merchantId ?: $xfOptions->tcPaygateHeleket_MerchantId;
		$apiKey = $apiKey ?: $xfOptions->tcPaygateHeleket_ApiKey;

		$dataString = json_encode($data, JSON_UNESCAPED_UNICODE);

		$httpClient = new GuzzleHttp\Client();
		try
		{
			$response = $httpClient->post($this->getApiEndpoint() . $path, [
				'headers' => [
					'merchant'     => $merchantId,
					'sign'         => $this->getSignature($dataString, $apiKey),
					'Content-Type' => 'application/json'
				],
				'body'    => $dataString
			]);
		}
		catch (RequestException $e)
		{
			XenForo_Error::logException($e);

			if ($e->hasResponse() && $e->getResponse()->getStatusCode() != 401)
			{
				$responseData = json_decode($e->getResponse()->getBody(), true);

				$error = $responseData['message'] ?? new XenForo_Phrase('something_went_wrong_please_try_again');

				return [];
			}

			$error = new XenForo_Phrase('something_went_wrong_please_try_again');

			return [];
		}

		$responseData = json_decode($response->getBody(), true);

		$state = $responseData['state'] ?? 0;
		if (is_array($responseData) && $state == 0)
		{
			return $responseData['result'] ?? [];
		}
		else
		{
			if (!empty($responseData['message']))
			{
				$error = $responseData['message'];

				return [];
			}

			if (!empty($responseData['errors']))
			{
				$error = is_array($responseData['errors']) ? json_encode($responseData['errors']) : $responseData['errors'];

				return [];
			}
		}

		$error = new XenForo_Phrase('something_went_wrong_please_try_again');

		return [];
	}

	/**
	 * @param string $data
	 * @param string $apiKey
	 *
	 * @return string
	 */
	public function getSignature($data, $apiKey = null)
	{
		$xfOptions = XenForo_Application::getOptions();
		$apiKey = $apiKey ?: $xfOptions->tcPaygateHeleket_ApiKey;

		return hash('md5', base64_encode($data) . $apiKey);
	}

	/**
	 * @param $network
	 * @param $address
	 * @param $amount
	 *
	 * @return string
	 */
	public function generatePaymentLink($network, $address, $amount)
	{
		if ($network == 'ton')
		{
			$amount = number_format($amount, 9, '', '');

			return 'https://app.tonkeeper.com/transfer/' . $address . '?amount=' . $amount;
		}

		return $network . ':' . $address . '?amount=' . $amount;
	}

	/**
	 * @param string $message
	 * @param string $type
	 *
	 * @return false|string
	 */
	protected function getResponse($message = '', $type = 'result')
	{
		return json_encode([
			$type => [
				'message' => $message
			]
		]);
	}
}